-- schema_update_1_1_0
-- This file contains incrementation schema changes to upgrade an existing Whitebeam
-- schema to release 1.1.0. The file *only* works if applied to a version of the schema
-- from version 0.9.37 of Whitebeam.
--
-- Attach to your Whitebeam database with 'psql' and issue the command :
-- \i schema_upgrade_1_10
--
-- Don't forget to run createlang:
-- eg.     createlang -Upostgres -dwhitebeam plpgsql
--
DROP SEQUENCE  collection_id_seq;
DROP INDEX collection_left;
DROP INDEX collection_right;

DROP RULE  clean_collections_c ON contacts;
DROP RULE  clean_collections_i ON items;
DROP RULE  clean_collections_n ON searchtree;

DROP TABLE collections CASCADE;
DROP TABLE collection_types CASCADE;

CREATE SEQUENCE collection_id_seq START 100 INCREMENT 1 CACHE 50;      

CREATE TABLE collection_types (
  instance CHAR(1) NOT NULL,
  client_id VARCHAR(30) NOT NULL,
  collection_id BIGINT DEFAULT nextval('collection_id_seq'::text),
  name VARCHAR(50) NOT NULL,
  opaque_data VARCHAR,

  CONSTRAINT con_collection_type_pk PRIMARY KEY (instance,client_id,collection_id,name),
  CONSTRAINT con_collection_type_unique UNIQUE (instance,client_id,name),
  CONSTRAINT con_collection_type_unique2 UNIQUE (instance,client_id,collection_id)
)
WITHOUT OIDS;

CREATE TABLE collections (
  instance CHAR(1) NOT NULL,
  client_id VARCHAR(30) NOT NULL,
  collection_id bigint NOT NULL,

  -- Specify the two end points for the relationship
  left_type CHAR(1) NOT NULL,
  left_id   BIGINT  NOT NULL,

  right_type CHAR(1) NOT NULL,
  right_id   BIGINT  NOT NULL,

  sub_type VARCHAR(30),

  -- Data associated with the arc between the two end-points
  custom_data VARCHAR,

  -- Store a modification date whch can be used as the basis of a sort order
  recorded_time TIMESTAMP,

  -- Rule is that there can only be ONE arc for each collection type between two end-points.
  CONSTRAINT con_collection_pk PRIMARY KEY (instance,client_id,collection_id,left_type,left_id,right_type,right_id)
)
WITHOUT OIDS;

-- Create source and target indices
CREATE INDEX collection_left ON collections USING BTREE (instance,client_id,left_type,left_id);
CREATE INDEX collection_right ON collections USING BTREE (instance,client_id,right_type,right_id);

-- Add deletion rules to the contact and catalogue tables to make sure we tidy up the 
-- collections when elements are deleted
CREATE OR REPLACE RULE clean_collections_c AS
    ON DELETE TO contacts DO  
  DELETE FROM collections
    WHERE old.instance = collections.instance AND 
          old.client_id::text = collections.client_id::text AND 
          collections.left_type = 'C'::bpchar AND 
          old.contact_id = collections.left_id 
     OR 
          old.instance = collections.instance AND 
          old.client_id::text = collections.client_id::text AND 
          collections.right_type = 'C'::bpchar AND 
          old.contact_id = collections.right_id;

CREATE OR REPLACE RULE clean_collections_i AS
    ON DELETE TO items DO  
  DELETE FROM collections
    WHERE old.instance = collections.instance AND 
          old.client_id::text = collections.client_id::text AND 
          collections.left_type = 'I'::bpchar AND 
          old.item_id = collections.left_id 
     OR 
          old.instance = collections.instance AND 
          old.client_id::text = collections.client_id::text AND 
          collections.right_type = 'I'::bpchar AND 
          old.item_id = collections.right_id;

CREATE OR REPLACE RULE clean_collections_n AS
    ON DELETE TO searchtree DO  
  DELETE FROM collections
    WHERE old.instance = collections.instance AND 
          old.client_id::text = collections.client_id::text AND 
          collections.left_type = 'I'::bpchar AND 
          old.node_id = collections.left_id 
     OR 
          old.instance = collections.instance AND 
          old.client_id::text = collections.client_id::text AND 
          collections.right_type = 'I'::bpchar AND 
          old.node_id = collections.right_id;

-- Rule to remove all entities in a collection class if that class itself is removed
CREATE OR REPLACE RULE clean_collections AS
    ON DELETE TO collection_types DO
  DELETE FROM collections
    WHERE old.instance=collections.instance AND
          old.client_id=collections.client_id AND
          old.collection_id=collections.collection_id;


-- The following declarations replace ineffecient indices on several tables with ones
-- that appear to product significantly better query results.

-- Firstly two indices on the 'contacts' table. Qualify with instance and client_id
DROP INDEX co_pa_ind;
DROP INDEX co_ty_ind;
CREATE INDEX co_pa_ind ON CONTACTS (instance,client_id,parent);
CREATE INDEX co_ty_ind ON CONTACTS (instance,client_id,"type");

-- Then 'items' from the contact template
DROP INDEX it_co_ind;
CREATE INDEX it_co_ind ON ITEMS (instance,client_id,container_mkr);

-- And the searchtree
DROP INDEX se_ty_ind;
CREATE INDEX se_ty_ind ON SEARCHTREE (instance,client_id,type_id);

-- Add in our search ranking function
CREATE OR REPLACE FUNCTION matchString(searchStr text,srcStr text) RETURNS INTEGER AS $$
DECLARE
        count integer := 0;
        keywords text[] := string_to_array(trim(lower(searchStr)),' ');
        str text := ' ' || lower(srcStr) || ' ';
BEGIN
        IF str<>'' THEN
                FOR idx IN 1..array_upper(keywords,1) LOOP
                        IF str LIKE '%'||keywords[idx]||'%'
                        THEN
                                count := count+1;
                        END IF;
                END LOOP;
        END IF;
        RETURN count;
END;
$$ LANGUAGE plpgsql;

ALTER TABLE contacts ADD COLUMN value BIGINT;
ALTER TABLE contacts ALTER COLUMN value SET DEFAULT 0;
UPDATE contacts SET value=0 WHERE value IS NULL;
ALTER TABLE contacts ALTER COLUMN value SET NOT NULL;
ALTER TABLE contacts ADD COLUMN keywords VARCHAR(1000);
ALTER TABLE contacts ALTER COLUMN description TYPE VARCHAR(1000);
